"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var EyeglassCompiler = require("broccoli-eyeglass");
var Eyeglass = require("eyeglass");
var findHost_1 = require("./findHost");
var MergeTrees = require("broccoli-merge-trees");
var path = require("path");
var url = require("url");
var cloneDeep = require("lodash.clonedeep");
var defaultsDeep = require("lodash.defaultsdeep");
var broccoli_ln_s_1 = require("./broccoli-ln-s");
var EYEGLASS_INFO_PER_ADDON = new WeakMap();
var EYEGLASS_INFO_PER_APP = new WeakMap();
var apps = new Array();
//eslint-disable-next-line @typescript-eslint/no-explicit-any
function isLazyEngine(addon) {
    if (addon.lazyLoading === true) {
        // pre-ember-engines 0.5.6 lazyLoading flag
        return true;
    }
    if (addon.lazyLoading && addon.lazyLoading.enabled === true) {
        return true;
    }
    return false;
}
//eslint-disable-next-line @typescript-eslint/no-explicit-any
function getDefaultAssetHttpPrefix(parent) {
    // the default http prefix differs between Ember app and lazy Ember engine
    // iterate over the parent's chain and look for a lazy engine or there are
    // no more parents, which means we've reached the Ember app project
    var current = parent;
    while (current.parent) {
        if (isLazyEngine(current)) {
            // only lazy engines will inline their assets in the engines-dist folder
            return "/engines-dist/" + current.name + "/assets";
        }
        current = current.parent;
    }
    // at this point, the highlevel container is Ember app and we should use the default 'assets' prefix
    return 'assets';
}
/* addon.addons forms a tree(graph?) of addon objects that allow us to traverse the
 * ember addon dependencies.  However there's no path information in the addon object,
 * but each addon object has some disconnected metadata in addon.addonPackages
 * with the path info. Unfortunately there's no shared information that would
 * allow us to connect which addon packages are actually which addon objects.
 * It would be great if ember-cli didn't throw that information on the ground
 * while building these objects. It would also be marvelous if we knew which
 * addons came from a local addon declaration and which ones came from node
 * modules.
 **/
function localEyeglassAddons(addon) {
    var paths = new Array();
    if (typeof addon.addons !== 'object' ||
        typeof addon.addonPackages !== 'object') {
        return paths;
    }
    var packages = Object.keys(addon.addonPackages);
    for (var i = 0; i < packages.length; i++) {
        var p = addon.addonPackages[packages[i]];
        // Note: this will end up creating manual addons for things in node modules
        // that are actually auto discovered, these manual modules will get deduped
        // out.  but we need to add all of them because the some eyeglass modules
        // for addons & engines won't get autodiscovered otherwise unless the
        // addons/engines are themselves eyeglass modules (which we don't want to require).
        if (p.pkg.keywords.some(function (kw) { return kw == 'eyeglass-module'; })) {
            paths.push({ path: p.path });
        }
    }
    // TODO: if there's a cycle in the addon graph don't recurse.
    for (var i = 0; i < addon.addons.length; i++) {
        paths = paths.concat(localEyeglassAddons(addon.addons[i]));
    }
    return paths;
}
var EMBER_CLI_EYEGLASS = {
    name: 'ember-cli-eyeglass',
    included: function (parent) {
        this._super.included.apply(this, arguments);
        this.initSelf();
    },
    initSelf: function () {
        if (EYEGLASS_INFO_PER_ADDON.has(this))
            return;
        var app = findHost_1.default(this);
        if (!app)
            return;
        var isApp = (this.app === app);
        var name = app.name;
        if (!isApp) {
            var thisName = typeof this.parent.name === "function" ? this.parent.name() : this.parent.name;
            name = name + "/" + thisName;
        }
        var parentPath = this.parent.root;
        if (isApp) {
            apps.push(app);
            // we create the symlinker in persistent mode because there's not a good
            // way yet to recreate the symlinks when sass files are cached. I would
            // worry about it more but it seems like the dist directory is cumulative
            // across builds anyway.
            EYEGLASS_INFO_PER_APP.set(app, {
                sessionCache: new Map(),
                assets: new broccoli_ln_s_1.BroccoliSymbolicLinker({}, { annotation: app.name, persistentOutput: true })
            });
        }
        var addonInfo = { isApp: isApp, name: name, parentPath: parentPath, app: app };
        EYEGLASS_INFO_PER_ADDON.set(this, addonInfo);
    },
    postBuild: function (result) {
        var e_1, _a;
        Eyeglass.resetGlobalCaches();
        try {
            for (var apps_1 = __values(apps), apps_1_1 = apps_1.next(); !apps_1_1.done; apps_1_1 = apps_1.next()) {
                var app = apps_1_1.value;
                var appInfo = EYEGLASS_INFO_PER_APP.get(app);
                if (appInfo) {
                    appInfo.assets.reset();
                    appInfo.sessionCache.clear();
                }
                else {
                    // eslint-disable-next-line no-console
                    console.warn("eyeglass app info is missing during postBuild.");
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (apps_1_1 && !apps_1_1.done && (_a = apps_1.return)) _a.call(apps_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    },
    postprocessTree: function (type, tree) {
        var addonInfo = EYEGLASS_INFO_PER_ADDON.get(this);
        if (type === "all" && addonInfo.isApp) {
            var appInfo = EYEGLASS_INFO_PER_APP.get(addonInfo.app);
            return new MergeTrees([tree, appInfo.assets], { overwrite: true });
        }
        else {
            return tree;
        }
    },
    setupPreprocessorRegistry: function (type, registry) {
        var _this = this;
        var addon = this;
        registry.add('css', {
            name: 'eyeglass',
            ext: 'scss',
            toTree: function (tree, inputPath, outputPath) {
                // These start with a slash and that messes things up.
                var cssDir = outputPath.slice(1) || './';
                var sassDir = inputPath.slice(1) || './';
                var app = EYEGLASS_INFO_PER_ADDON.get(_this).app;
                var extracted = _this.extractConfig(app, addon);
                extracted.cssDir = cssDir;
                extracted.sassDir = sassDir;
                var config = _this.setupConfig(extracted);
                var compiler = new EyeglassCompiler(tree, config);
                compiler.events.on("cached-asset", function (absolutePathToSource, httpPathToOutput) {
                    _this.linkAsset(absolutePathToSource, httpPathToOutput);
                });
                return compiler;
            }
        });
    },
    extractConfig: function (host, addon) {
        var isNestedAddon = typeof addon.parent.parent === 'object';
        // setup eyeglass for this project's configuration
        var hostConfig = cloneDeep(host.options.eyeglass || {});
        var addonConfig = isNestedAddon ? cloneDeep(addon.parent.options.eyeglass || {}) : {};
        return defaultsDeep(addonConfig, hostConfig);
    },
    linkAsset: function (srcFile, destUri) {
        if (path.posix.isAbsolute(destUri)) {
            destUri = path.posix.relative("/", destUri);
        }
        if (process.platform === "win32") {
            destUri = url.fileURLToPath("file://" + destUri);
        }
        var app = EYEGLASS_INFO_PER_ADDON.get(this).app;
        var assets = EYEGLASS_INFO_PER_APP.get(app).assets;
        return assets.ln_s(srcFile, destUri);
    },
    setupConfig: function (config, options) {
        var _this = this;
        var _a = EYEGLASS_INFO_PER_ADDON.get(this), isApp = _a.isApp, app = _a.app, parentPath = _a.parentPath;
        var sessionCache = EYEGLASS_INFO_PER_APP.get(app).sessionCache;
        config.sessionCache = sessionCache;
        config.annotation = "EyeglassCompiler(" + parentPath + ")";
        if (!config.sourceFiles && !config.discover) {
            config.sourceFiles = [isApp ? 'app.scss' : 'addon.scss'];
        }
        config.assets = ['public', 'app'].concat(config.assets || []);
        config.eyeglass = config.eyeglass || {};
        // XXX We don't set the root anywhere but I'm not sure what might break if we do.
        // config.eyeglass.root = parentPath;
        config.eyeglass.httpRoot = config.eyeglass.httpRoot || config["httpRoot"];
        if (config.persistentCache) {
            var cacheDir = parentPath.replace(/\//g, "$");
            config.persistentCache += "/" + cacheDir;
        }
        config.assetsHttpPrefix = config.assetsHttpPrefix || getDefaultAssetHttpPrefix(this.parent);
        if (config.eyeglass.modules) {
            config.eyeglass.modules =
                config.eyeglass.modules.concat(localEyeglassAddons(this.parent));
        }
        else {
            config.eyeglass.modules = localEyeglassAddons(this.parent);
        }
        var originalConfigureEyeglass = config.configureEyeglass;
        config.configureEyeglass = function (eyeglass, sass, details) {
            eyeglass.assets.installer(function (file, uri, fallbackInstaller, cb) {
                cb(null, _this.linkAsset(file, uri));
            });
            if (originalConfigureEyeglass) {
                originalConfigureEyeglass(eyeglass, sass, details);
            }
        };
        // If building an app, rename app.css to <project>.css per Ember conventions.
        // Otherwise, we're building an addon, so rename addon.css to <name-of-addon>.css.
        var originalGenerator = config.optionsGenerator;
        config.optionsGenerator = function (sassFile, cssFile, sassOptions, compilationCallback) {
            if (isApp) {
                cssFile = cssFile.replace(/app\.css$/, _this.app.name + ".css");
            }
            else {
                cssFile = cssFile.replace(/addon\.css$/, _this.parent.name + ".css");
            }
            if (originalGenerator) {
                originalGenerator(sassFile, cssFile, sassOptions, compilationCallback);
            }
            else {
                compilationCallback(cssFile, sassOptions);
            }
        };
        return config;
    }
};
module.exports = EMBER_CLI_EYEGLASS;
//# sourceMappingURL=index.js.map